
//Global variables and tables

//List of stages whose skyboxes and weather we probably shouldn't be touching
rawset(_G, "xmasBlacklist", 
	{"egg rock", "black core", "azure temple", //Vanilla
     "midnight freeze", "sunken plant", "inner sanctum", "prismatic angel", "spatial void", "primordial abyss", "mystic realm", "dimension warp", //Mystic Realm
     "space walk"} //Tortured Planet
)
rawset(_G, "xmasBl", false)		//Bool checking if a christmas stage is blacklisted from the above list. Used by the HUD.
rawset(_G, "emblems", {})		//Stage goal emblems
rawset(_G, "checkpoints", {})	//Checkpoints
rawset(_G, "active", 1)			//How many emblems are left in this stage
rawset(_G, "mapSpawn", 0)		//Where player 1 should spawn in our current map
rawset(_G, "activateAll", true) //Should all previous checkpoints be activated when activating a new one?
rawset(_G, "KRIMA", 0)			//Was the christmas command used?
rawset(_G, "emDifficulty", 2)   //What emblems should be created from
rawset(_G, "superEH", false)    //Whether or not maps will start with all 7 emeralds

//3D distance used by getProximity
rawset(_G, "dist3D", function(mo1,mo2)
	local x = mo2.x - mo1.x
	local y = mo2.y - mo1.y
	local z = mo2.z - mo1.z
	return FixedHypot(FixedHypot(x,y),z)
end)

//Proximity checker for the emblem radar
rawset(_G, "getProximity", function(mo, target)
	if not mo.valid or not target.valid return 1 end
	local dist = dist3D(mo,target)/FRACUNIT
	if target.inactive return 1 end
	//Data taken from source code
	local i = 1
	if dist < 128
		i = 6
	elseif dist < 512
		i = 5
	elseif dist < 1024
		i = 4
	elseif dist < 2048
		i = 3
	elseif dist < 3072
		i = 2
	end
	return i
end)

//Angle checker for sorting checkpoints
rawset(_G, "angleCheck", function(a,b)
	return (a.spawnpoint.angle + a.spawnpoint.extrainfo*360) < (b.spawnpoint.angle + b.spawnpoint.extrainfo*360)
end)

//Convert strings to tables
rawset(_G, "stringToTable", function(str)
	local t = {}
	for i = 1, #str do t[i] = str:sub(i, i) end
	return t
end)

//Reset everything
rawset(_G, "allReset", function(a,b)
	mapSpawn = 0
	emblems = {}
	checkpoints = {}
	active = 1
	for p in players.iterate do
		p.warpTable = nil
		p.warpDest = nil
		p.checkWarp = nil
		p.warpMenu = false
		p.flashFade = false
		p.warpTimer = -1
	end
end)

//Function for creating 3D presents around mobjs
rawset(_G, "giftify", function(emblem)
	emblem.flags2 = $ | MF2_DONTDRAW
	emblem.gift = {}
	local giftColor = P_RandomRange(1,#skincolors-1)
	for i=1, 4
		//Four sides of our gift
		local giftSide = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGSIDE)
		giftSide.angle = ANGLE_90*i
		giftSide.target = emblem
		giftSide.scale = FRACUNIT/2
		giftSide.color = giftColor
		table.insert(emblem.gift, giftSide)
		if i>2
			//Top and bottom of our gift
			local giftTop = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGTOP)
			giftTop.target = emblem
			giftTop.renderflags = $ | RF_NOSPLATBILLBOARD
			giftTop.spriteyoffset = -30*FRACUNIT
			giftTop.top = i==3 and true or false
			giftTop.state = i==3 and S_GITO or S_GIBO
			giftTop.scale = FRACUNIT/2
			giftTop.color = giftColor
			table.insert(emblem.gift, giftTop)
		end
		if i==4
			//And finally a little ribbon
			local ribbon = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGRIB)
			ribbon.target = emblem
			ribbon.scale = 3*FRACUNIT/5
			ribbon.color = giftColor//ColorOpposite(giftColor)
			ribbon.shadowscale = 2*FRACUNIT
			table.insert(emblem.gift, ribbon)
		end
	end
end)