
//HERE BE SPAGHETTI

//Icon bobbing. Also used for the warp menu top text.
local bobRange = 3
local bobSpeed = 5
local bobDissonance = 18

//Variables that need to be initialized ASAP and preserved between loops
local bgScroll = 0
local alternator = false
local livesDisabled = false
local overlayColor = 253
local overlayIntensity = 5
local destHori = nil
local currHori = nil

//Tables for fade-ins and outs. Icons get their own table to fade faster but still sync with the warp background
local fadeTable = {V_90TRANS,V_80TRANS,V_70TRANS,V_60TRANS,V_50TRANS,V_40TRANS,V_30TRANS,V_20TRANS,V_10TRANS,V_HUDTRANS}
local IconFadeTable = {V_60TRANS,V_90TRANS,V_70TRANS,V_50TRANS,V_30TRANS,V_10TRANS,V_HUDTRANS}
local teleportFade = {1,2,3,4,5,5,4,3,2,1}

hud.add(function(v,p)
	if not (gametype == GT_EMBHUNT) return end
	if KRIMA and KRIMA == 3 and not xmasBl then v.fadeScreen(overlayColor, overlayIntensity) end
end,"scores")

hud.add(function(v,p)
	if not (gametype == GT_EMBHUNT) return end
	if KRIMA and KRIMA == 3 and not xmasBl then v.fadeScreen(overlayColor, overlayIntensity) end
end,"intermission")

//HUD stuff for emblems
hud.add(function(v,p)

	//Sanity checks
	if not (gametype == GT_EMBHUNT)
		if livesDisabled then hud.enable("lives") livesDisabled = false end
		return
	end
	if (p.mo == nil) or (not p.valid) then return end
	if not (emblems) then return end

	if KRIMA and KRIMA == 3 and not xmasBl then v.fadeScreen(overlayColor, overlayIntensity) end

	//Keep scrolling our background and alternating the checkpoints' flashing icons
	bgScroll = $-1
	if not (leveltime%5) then alternator = not alternator end

	//disable lives and add our sprites
	hud.disable("lives")
	livesDisabled = true
	v.gemb = v.cachePatch("goalemb")		//Small goal emblem icon
	v.warpbg = v.cachePatch("chkwabg")		//Warp screen background
	v.chkicou = v.cachePatch("chkiconu")	//Icon-sized checkpoints (unlit)
	v.chkicol = v.cachePatch("chkiconl")	//Icon-sized checkpoints (lit)
	v.chksel = v.cachePatch("chkselect")	//Warp menu selection box
	v.radar1 = v.cachePatch("HOMITM1")		//Emerald radar sprites
	v.radar2 = v.cachePatch("HOMITM2")		//         ||
	v.radar3 = v.cachePatch("HOMITM3")		//         ||
	v.radar4 = v.cachePatch("HOMITM4")		//         ||
	v.radar5 = v.cachePatch("HOMITM5")		//         ||
	v.radar6 = v.cachePatch("HOMITM6")		//         ||
	

	//Fade in and out for warps. I decided not to use P_FlashPal() to prevent eye strains from constant warping
	if p.warpTimer and p.warpTimer > 0 then v.fadeScreen(0, teleportFade[p.warpTimer]) end

	//Bobbing emblem sprites under RINGS. Only displayed if we're not in the warp menu
	//HORI and VERTI are in charge of positioning our emblems in a way that makes them
	//go to a lower row every 5 emblems
	if p.warpScreenFade <= 1
		local hori = 1
		local verti = 1
		//the for loop will position each emblem in our emblem table (which is being carried by our players)
		for i=1,#emblems do
			if not emblems[i].valid continue end
			//in case an emblem has been collected we give it the V_HUDTRANSHALF flag
			if hori == 6 then hori = 1 end //reseting HORI when appropriate
			//Ypos positions our emblems appropriately when we have multiples of 5
			local Ypos = (verti-1) * FRACUNIT * 13
			//iconbob bobs the icons. good old sine waves.
			local iconBob = sin( FixedAngle((leveltime+bobDissonance*i)*FRACUNIT*bobSpeed) ) * bobRange
			//finally we draw our emblems using the numbers we got above
			v.drawScaled(
				(hudinfo[HUD_RINGS].x) * FRACUNIT * hori,
				(hudinfo[HUD_RINGS].y+18) * FRACUNIT + Ypos + iconBob,
				FRACUNIT/2,
				v.gemb,
				emblems[i].inactive and V_PERPLAYER|V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANSHALF|V_PERPLAYER or V_PERPLAYER|V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS
			)
			hori = $+1
			if not (i%5) then verti = $+1 end
		end

		//Emblem radar. Also hidden when the menu is present.
		for i=1,#emblems do
			if not emblems[i].valid continue end
			local hori = (152 - 9*(#emblems-1)) + (18*(i-1))
			local radarColor = { v.radar1 , v.radar2 , v.radar3 , v.radar4 , v.radar5 , v.radar6 }
			local proxBeep = { 50, 50, 40, 20, 10, 5 }
			local proximity = getProximity(p.mo, emblems[i])
			if proximity > 1 and not (leveltime % proxBeep[proximity])
				S_StartSoundAtVolume(p.mo, sfx_emfind, 125, p)
			end
			v.draw(
				hori,
				165,
				emblems[i].inactive and v.radar1 or radarColor[proximity],
				emblems[i].inactive and V_PERPLAYER|V_SNAPTOBOTTOM|V_HUDTRANSHALF or V_PERPLAYER|V_SNAPTOBOTTOM|V_HUDTRANS
			)
		end
	end

	//Warp menu announcement
	if p.warpTextFade > 1 and not p.warpMenu
		v.drawString(
			160, 
			150, 
			"\x82Press \x88[TOSS FLAG]\x82 to open the warp menu", 
			V_PERPLAYER|V_SNAPTOBOTTOM|fadeTable[p.warpTextFade],
			"thin-center")
	end

	//Warp screen stuff
	if p.warpScreenFade > 1

		//Warp screen background
		v.draw(bgScroll,0,v.warpbg, V_PERPLAYER|V_SNAPTOLEFT|fadeTable[p.warpScreenFade])
		
		//Warp screen top text. Wibbly wobbly!
		local strTable = stringToTable("Warp Menu")
		for i=1,#strTable
			local bob = sin( FixedAngle((leveltime+(bobDissonance/2)*i)*FRACUNIT*bobSpeed) ) * bobRange
			local hori = (160 - 4*(#strTable-1)) + (8*(i-1))
			v.drawString(hori * FRACUNIT, 40 * FRACUNIT + bob, strTable[i], V_PERPLAYER|V_YELLOWMAP|IconFadeTable[p.warpScreenFade], "fixed-center")
		end
		
		//Horizontal and vertical positioning of checkpoint icons, same formula as the radar
		for i=1,#checkpoints do
			local flags = p.warpTable[i] and IconFadeTable[p.warpScreenFade] or fadeTable[p.warpScreenFade]
			local icon = v.chkicou
			if p.warpTable[i] and not alternator then icon = v.chkicol end
			if #checkpoints >= 8
				//ABANDON ALL HOPE
				local horiCompact = #checkpoints <= 15 and 
					(152 - 10*(#checkpoints-1))*FRACUNIT + (20*(i-1))*FRACUNIT + 3*(FRACUNIT/2) or
					(152 - 10 * 14)*FRACUNIT + (20*(i-1))*FRACUNIT + 3*(FRACUNIT/2)
				local Ypos = #checkpoints <= 15 and
					1 or 
					((i-1)/15) * 67*FRACUNIT - 25*FRACUNIT*(  ((i-1)/15)+1  )
				//Draw checkpoints
				v.drawScaled( horiCompact - 300*FRACUNIT*((i-1)/15) , 85*FRACUNIT + Ypos , FRACUNIT/2, icon , flags|V_PERPLAYER )
				//Draw checkpoint text
				v.drawString((horiCompact/FRACUNIT) + 4 - 300*((i-1)/15), 120 + (Ypos/FRACUNIT), "#"+i, flags|V_PERPLAYER|V_YELLOWMAP, "thin-center")
			else
				local horiBig = (152 - 18*(#checkpoints-1)) + (36*(i-1))
				//Draw checkpoints
				v.draw( horiBig , 60 , icon , flags|V_PERPLAYER )
				//Draw checkpoint text
				v.drawString(horiBig+9, 129, "#"+i, flags|V_PERPLAYER|V_YELLOWMAP, "center")
			end
		end

		//Draw checkpoint selector and lerp it to the right place
		//the lerp is done by creating CURRHORI and DESTHORI as local vars outside this if
		//DESTHORI is updated immediately, while CURRHORI is updated in increments towards DESTHORI
		//afterwards we send CURRHORI to our final HORI, which becomes our X position
		if #checkpoints >= 8
			destHori = #checkpoints <= 15 and 
				( (152 - 10*(#checkpoints-1)) + (20*(p.warpCursor-1)) ) * FRACUNIT - 5 * (FRACUNIT/2) or
				( (152 - 10 * 14) + (20*(p.warpCursor-1)) ) * FRACUNIT - 5 * (FRACUNIT/2)
			local Ypos = #checkpoints <= 15 and
				1 or 
				((p.warpCursor-1)/15) * 67*FRACUNIT - 25*FRACUNIT*(  ((p.warpCursor-1)/15)+1  )
			if not currHori or p.warpScreenFade == 2 then currHori = destHori end
			local hori = ease.insine(FRACUNIT/2, currHori, destHori)
			currHori = hori
			v.drawScaled(hori - 300*FRACUNIT*((p.warpCursor-1)/15) , 83*FRACUNIT + Ypos, FRACUNIT/2, v.chksel, V_PERPLAYER|IconFadeTable[p.warpScreenFade])
		else
			destHori = ( (144 - 18*(#checkpoints-1)) + (36*(p.warpCursor-1)) ) * FRACUNIT
			if not currHori or p.warpScreenFade == 2 then currHori = destHori end
			local hori = ease.insine(FRACUNIT/2, currHori, destHori)
			currHori = hori
			v.drawScaled(hori, 56*FRACUNIT, FRACUNIT, v.chksel, V_PERPLAYER|IconFadeTable[p.warpScreenFade])
		end

		//Warp screen instructions
		local flags = IconFadeTable[p.warpScreenFade]
		v.drawString(160, 155, "\x82Press \x88[JUMP] \x82to warp to an active checkpoint", flags|V_PERPLAYER, "thin-center")
		v.drawString(160, 170, "\x82Press \x88[SPIN] \x82to warp back to the start of the level", flags|V_PERPLAYER, "thin-center")
		v.drawString(160, 185, "\x82Press \x88[TOSS FLAG] \x82to close the warp menu", flags|V_PERPLAYER, "thin-center")
	end

	//Resetting our background's scroll position after it scrolls enough
	if bgScroll == -140 then bgScroll = 1 end

end,"game")