
//Resetting stuff on map changes
addHook("MapChange", function(mapnum)
	if not (gametype == GT_EMBHUNT) return end
	allReset()
end)

//Let's find the map's default emblems
addHook("MapLoad", function()
	if not (gametype == GT_EMBHUNT) return end
	allReset()
	local id = 1
	if superEH then emeralds = EMERALD1|EMERALD2|EMERALD3|EMERALD4|EMERALD5|EMERALD6|EMERALD7
	else emeralds = 0 end

	//Iterating through things to find relevant objects
	for em in mapthings.iterate do
		//Finding vanilla emblems or emerald tokens and adding them to our table
		if (em.type == mobjinfo[MT_EMBLEM].doomednum and (emDifficulty == 2 or emDifficulty == 3))
		or (em.type == mobjinfo[MT_TOKEN].doomednum and (emDifficulty == 1 or emDifficulty == 3))
			if (not R_PointInSubsectorOrNil(em.x*FRACUNIT, em.y*FRACUNIT)) or (em.x == 32767 and em.y == 32767) continue end //We found a emblem out of bounds! Skip this one...
			local goal = P_SpawnMobj(em.x*FRACUNIT, em.y*FRACUNIT, 0, MT_GOALEMBLEM)
			if P_CheckDeathPitCollide(goal) then goal.z = P_FloorzAtPos(em.x*FRACUNIT, em.y*FRACUNIT, em.z*FRACUNIT, 16*FRACUNIT) + FRACUNIT end
			goal.color = 64
			goal.shadowscale = FRACUNIT
			goal.zdelay = 2 //solution to Z issue taken from shine's Adventure Emblems
			goal.zdest = em.z*FRACUNIT
			goal.id = id
			table.insert(emblems, goal)
			id = $+1
		end

		//Reducing 1-Up boxes to atoms
		if em.type == mobjinfo[MT_1UP_BOX].doomednum then P_RemoveMobj(em.mobj) end
		//Adding P1's spawn position as the checkpoint warp's warp position
		if em.type == 1 then mapSpawn = em end
	end
	//If no emblem was found, place an emblem at the stage end sign
	local activateSpecial = false
	if not #emblems
		for sign in mapthings.iterate do
			if sign.type == mobjinfo[MT_SIGN].doomednum
				if activateSpecial then continue end
				local goal = P_SpawnMobj(sign.x*FRACUNIT, sign.y*FRACUNIT, 0, MT_GOALEMBLEM)
				goal.zdelay = 2
				goal.zdest = sign.z
				goal.color = 64
				goal.shadowscale = FRACUNIT
				goal.signEmblem = true //marking our emblem as a special one. if any of these is approached, the level ends.
				table.insert(emblems, goal)
				activateSpecial = true
			end
		end
	end

	if activateSpecial 
		active = -1
	elseif not #emblems 
		active = 0 
	else 
		active = #emblems 
	end

	//IT'S CHRISTMAAAAAS! Let's make a rotating 3D present!
	if KRIMA and #emblems
		for i=1,#emblems do giftify(emblems[i]) end
	end

	//Christmas weather, night sky and color tint! Let's check our blacklist before applying them.
	if KRIMA and KRIMA >= 2
		xmasBl = false
		for i=0,#xmasBlacklist
			if string.lower(mapheaderinfo[gamemap].lvlttl) == xmasBlacklist[i] then xmasBl = true end
		end
		if not xmasBl
			P_SetupLevelSky(6)
			P_SetSkyboxMobj(nil)
			P_SwitchWeather(2)
		end
	end

	//Finding star posts for the warp system, then sorting them by order
	for mo in mobjs.iterate() do
		if mo.info == mobjinfo[MT_STARPOST] table.insert(checkpoints, mo) end
	end
	table.sort(checkpoints, angleCheck)

	//Give each valid player a sorted list of checkpoints
	//Invalid players can get checkpoints{} later in their playerThink when they become valid
	for p in players.iterate
		if not p.valid continue end
		p.warpTable = {}
		for i=1,#checkpoints do table.insert(p.warpTable, false) end
	end
end)

//An emblem was touched. Let's do some game mode stuff.
addHook("TouchSpecial", function(mo, toucher)
	if not toucher.valid return end
	if not toucher.player or not toucher.player.valid return end
	//Spawn some sparks when collecting the emblem, make it transparent, decrease total emblems and deactivate it
	P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_SPARK)
	mo.frame = $ | TR_TRANS50
	mo.inactive = true
	active = $-1
	mo.flags = $ & ~MF_SPECIAL
	//Is our emblem a gift? If so, make all of its parts transparent
	if mo.gift
		for i=1,table.maxn(mo.gift) do mo.gift[i].frame = $ | TR_TRANS50 end
	end
	//Default variables for emblem collection. Changed in christmas mode
	local embName = "emblem"
	local embSound = sfx_eget
	local embSoundEnd = sfx_eall
	//Christmas sounds and text!
	if KRIMA
		embName = "present"   embSound = sfx_nxitem   embSoundEnd = sfx_xideya
	end
	//Print a text for everyone, play a nice sound and end the function
	if mo.id > 0
		print("\x82- " + toucher.player.name + " has found " + embName + " #" + mo.id + "! - ")
	else
		print("\x82- " + toucher.player.name + " has reached the goal " + embName + "! - ")
	end
	if active == 0 then S_StartSound(nil, embSoundEnd)
	else S_StartSound(nil, embSound) end
	return true
end, MT_GOALEMBLEM)

//Menu input stuff goes here, like disabling a player's movement, jump and spin
addHook("PreThinkFrame", function()
	if not (gametype == GT_EMBHUNT) return end
	for p in players.iterate
		if not p.valid then continue end

		//Initialize some player variables
		p.warpMenu = $ or false
		if not p.warpScreenFade then p.warpScreenFade = 1 end
		
		//The warp menu is open, so let's freeze our character and do warp menu stuff
		if p.warpMenu

			//Preventing the PRESS TOSSFLAG text from reappearing after exiting the menu unless we walk away from the checkpoint
			p.menuTextStopper = true
			
			//Freeze our jump entirely
			p.pflags = $ | PF_JUMPSTASIS

			//Control the warp screen's fade in
			if p.warpScreenFade < 7 then p.warpScreenFade = $+1 end
			
			//We're trying to move. let's hijack that to transform it into a menu input.
			if p.cmd.sidemove
				if not p.sideMenuPress
					//If we pressed either left or right this frame...
					if p.cmd.sidemove < 0 and p.warpCursor >= 1
						p.warpCursor = $-1
						if p.warpCursor == 0 then p.warpCursor = #checkpoints end
						S_StartSoundAtVolume(p.mo, sfx_kc56, 200, p)
					elseif p.cmd.sidemove > 0 and p.warpCursor <= #checkpoints
						p.warpCursor = $+1
						if p.warpCursor == #checkpoints+1 then p.warpCursor = 1 end
						S_StartSoundAtVolume(p.mo, sfx_kc56, 200, p)
					end

				end
				p.sideMenuPress = true
			else
				p.sideMenuPress = false
			end

			if p.cmd.buttons & BT_JUMP
				if not p.jumpMenuPress
					//If we pressed jump this frame...
					if p.warpTable[p.warpCursor]
						if not (checkpoints[p.warpCursor] == p.checkWarp)
							p.warpDest = checkpoints[p.warpCursor]
							S_StartSound(p.mo, sfx_s3k54, p)
							p.warpTimer = 10
							p.warpScreenFade = 1
						else
							S_StartSound(p.mo, sfx_s3k76, p)
						end
						p.warpMenu = false
					else
						S_StartSound(p.mo, sfx_lose, p)
					end
				end
				p.jumpMenuPress = true
			else
				p.jumpMenuPress = false
			end

			if p.cmd.buttons & BT_SPIN
				if not p.spinMenuPress
					p.spawnWarp = true
					S_StartSound(p.mo, sfx_s3k54, p)
					p.warpTimer = 10
					p.warpScreenFade = 1
					p.warpMenu = false
				end
				p.spinMenuPress = true
			else
				p.spinMenuPress = false
			end

			//We're all done, so now let's freeze our movement
			p.cmd.buttons = $ & ~BT_SPIN
			p.cmd.sidemove = 0
			p.cmd.forwardmove = 0
		else
			if p.warpScreenFade > 1 then p.warpScreenFade = $-1 end
		end

	end
end)

//Players checking if the stage should be completed and sending emblem data to the HUD
addHook("PlayerThink", function(p)
	if not (gametype == GT_EMBHUNT) return end
	if not p.valid then return end

	//If we for some reason do not have a valid checkpoint list on map load, get a default one from our local table
	if p.warpTable == nil and not (checkpoints == {})
		p.warpTable = {}
		for i=1,#checkpoints do table.insert(p.warpTable, false) end
	end

	//Initialize more player variables
	if p.warpCursor == nil then p.warpCursor = 1 end
	if not p.warpTextFade then p.warpTextFade = 1 end
	//if p.destHori == nil then p.destHori = 0 end
	//if p.currHori == nil then p.currHori = 0 end
	
	//Exit the level when there are no longer any active emblems
	if ((active == 0 or active == -2) or (active == -1 and GetSecSpecial(p.mo.subsector.sector.special, 4) == 2))
	and not p.exiting
		if active == -1 then active = -2 end
		local embSoundEnd = sfx_eall
		if KRIMA then embSoundEnd = sfx_xideya end
		S_StartSound(p.mo, embSoundEnd, p) 
		P_DoPlayerFinish(p)
	end

	//Reset relevant warp menu variables if not close to a checkpoint or if it is invalid
	if p.checkWarp and ( not p.checkWarp.valid or getProximity(p.mo, p.checkWarp) < 6 )
		p.checkWarp = nil
		p.warpMenu = false
	end
	
	//If we are close to a star post and pressed toss flag this frame, enable the warp menu
	if p.checkWarp
		if p.warpTextFade < 10 and not p.menuTextStopper then p.warpTextFade = $+1 end
		if p.cmd.buttons & BT_TOSSFLAG and p.panim == PA_IDLE and P_IsObjectOnGround(p.mo)
			if not p.tossPress
				p.warpMenu = not p.warpMenu
				//Since we are enabling the warp menu, move our cursor to the checkpoint we are at
				if p.warpMenu
					for i=1, #checkpoints
						if checkpoints[i] == p.checkWarp then p.warpCursor = i end
					end
				else
					S_StartSound(p.mo, sfx_s3k76, p)
				end
			end
			p.tossPress = true
		else
			p.tossPress = false
		end
	else
		p.menuTextStopper = false
		if p.warpTextFade > 1 then p.warpTextFade = $-1 end
	end
	
	//Preventing the PRESS TOSSFLAG text from reappearing after exiting the menu unless we walk away from the checkpoint
	if p.menuTextStopper then p.warpTextFade = 1 end

	//Cancel our warp menu if we're not on the ground
	if p.warpMenu and not P_IsObjectOnGround(p.mo)
		p.warpMenu = false
	end

	//Preventing jumps and spins when exiting the warp menu
	if p.cmd.buttons & BT_JUMP and p.jumpMenuPress
		p.pflags = $ | PF_JUMPSTASIS
	elseif p.jumpMenuPress and not (p.cmd.buttons & BT_JUMP)
		p.jumpMenuPress = false
	end
	if p.spinMenuPress and p.spawnWarp
		p.pflags = $ | PF_FULLSTASIS
	elseif p.spinMenuPress and not p.spawnWarp
		p.spinMenuPress = false
	end

	//Time to warp! Count a few frames, then warp us to our checkpoint
	if p.warpTimer and p.warpTimer > 0
		p.warpTimer = $-1
		if p.warpTimer == 5
			if p.spawnWarp
				local floorZ = P_FloorzAtPos(mapSpawn.x*FRACUNIT, mapSpawn.y*FRACUNIT, mapSpawn.z*FRACUNIT, p.mo.height)
				P_TeleportMove(p.mo, mapSpawn.x*FRACUNIT, mapSpawn.y*FRACUNIT, floorZ + 75*FRACUNIT)
				if mapSpawn.options & MTF_OBJECTSPECIAL then p.mo.flags2 = $ | MF2_OBJECTFLIP else p.mo.flags2 = $ & ~MF2_OBJECTFLIP end
				if mapSpawn.angle then p.mo.angle = mapSpawn.angle end
			else 
				if p.warpDest.flags2 & MF2_OBJECTFLIP then p.mo.flags2 = $ | MF2_OBJECTFLIP else p.mo.flags2 = $ & ~MF2_OBJECTFLIP end
				P_TeleportMove(p.mo, p.warpDest.x + 4*FRACUNIT, p.warpDest.y + 4*FRACUNIT, p.warpDest.z + 75*FRACUNIT) 
				if p.warpDest.angle then p.mo.angle = p.warpDest.angle end
			end
			p.mo.flags2 = $ & ~MF2_TWOD
			p.spawnWarp = false
			p.mo.state = S_PLAY_FALL
		end
	end

end)

//A player was hit while menuing! Hide the menu!
local function damageMenuReset(mo)
	if not (gametype == GT_EMBHUNT) return end
	if not mo.valid or not mo.player.valid return end
	if mo.player.warpMenu then mo.player.warpMenu = false end
end

addHook("MobjDamage", damageMenuReset, MT_PLAYER)
addHook("MobjDeath", damageMenuReset, MT_PLAYER)

//A boss was defeated, even though I very specifically blacklisted bosses. Oh boy. Let's give 'em an emblem.
addHook("BossDeath", function(mo)
	if not (gametype == GT_EMBHUNT) return end
	if not mo.valid return end
	if mo.spawnedEmblem return end
	if mapheaderinfo[gamemap].lvlttl == "Black Core"
		active = 0 //black core keeps causing me headaches, so screw it. to the shadow realm with the metal sonic post-fight.
		return
	end
	local goal = P_SpawnMobj(mo.x, mo.y, mo.z, MT_GOALEMBLEM)
	goal.zdelay = 1
	goal.zdest = mo.z/FRACUNIT
	goal.color = 64
	goal.shadowscale = FRACUNIT
	mo.spawnedEmblem = true
	if KRIMA giftify(goal) end
end)

//Stuff to be synced between players
addHook("NetVars", function(net)
	emblems = net(emblems)
	active = net(active)
	checkpoints = net(checkpoints)
	activateAll = net(activateAll)
	KRIMA = net(KRIMA)
	emDifficulty = net(emDifficulty)
	xmasBl = net(xmasBl)
	if not xmasBl then P_SetSkyboxMobj(nil) end
end)