
//Variables for tweaking ease
local bobRange = 4
local bobSpeed = 3
local sparkleMult = FRACUNIT/2
local sparkleFrequency = 2 //the higher, the less sparkles
local rotSpeed = 3

//Function that creates and spreads sparkles over an area
local function CreateSparkle(mo)
	local sparkle = P_SpawnMobjFromMobj(mo, 0, 0, mo.height , MT_NIGHTSPARKLE)
	if KRIMA
		sparkle.colorized = true
		sparkle.color = SKINCOLOR_KETCHUP
	end
	sparkle.momx = P_RandomRange(-3, 3) * sparkleMult
	sparkle.momy = P_RandomRange(-3, 3) * sparkleMult
	sparkle.momz = P_RandomRange(-3, 3) * sparkleMult
	sparkle.scale = KRIMA and 2*FRACUNIT or 3*FRACUNIT/2
	sparkle.scalespeed = FRACUNIT/24
	sparkle.destscale = 1
	sparkle.frame = $ | TR_TRANS30
	sparkle.flags = MF_SCENERY|MF_NOCLIP|MF_NOCLIPHEIGHT
end

//Making our emblems bob up and down while "spinning"
addHook("MobjThinker", function(mo)
	if not mo.valid return end
	mo.id = $ or -1
	if mo.zdelay //solution to emblem Z positioning issue taken from shine's Adventure Emblems
		mo.zdelay = $-1
		if not mo.zdelay
			if mo.zdest != nil
				if not mo.subsector.sector.f_slope
					mo.z = mo.subsector.sector.floorheight+(mo.zdest)+16*FRACUNIT
				else
					mo.z = P_GetZAt(mo.subsector.sector.f_slope, mo.x, mo.y)+(mo.zdest)+16*FRACUNIT
				end
				mo.initialZ = mo.z
				mo.zdest = nil
			end
			mo.flags = MF_NOGRAVITY|MF_NOCLIPHEIGHT|MF_SPECIAL
			if mo.signEmblem then mo.flags = $ & ~MF_SPECIAL end
		end
	end
	mo.initialZ = $ or mo.z
	mo.z = mo.initialZ + sin( FixedAngle(leveltime*FRACUNIT*bobSpeed) ) * bobRange
	mo.angle = $+FixedAngle(rotSpeed*FRACUNIT)
	//Sparkles!
	if not (leveltime%sparkleFrequency) and not mo.inactive
		CreateSparkle(mo)
	end
end, MT_GOALEMBLEM)

//Thinker for star posts
addHook("MobjThinker", function(mo)
	if not mo.valid return end

	for p in players.iterate do
		if not p.valid or not p.mo.valid continue end

		//If a star post is active and a player is nearby, activate that star post (and previous ones) for warping
		if getProximity(mo, p.mo) == 6
			p.checkWarp = mo
			if p.warpTable
				local warpPos = -1
				for i=1,#checkpoints do if checkpoints[i] == mo then warpPos = i end end
				//Either activate all previous checkpoints or just this one
				if activateAll
					for i=1,warpPos do p.warpTable[i] = true end
				else
					p.warpTable[warpPos] = true
				end
			end
		end
	end
end, MT_STARPOST)

addHook("MobjThinker", function(mo)
	if not mo.valid return end
	mo.angle = $+FixedAngle(rotSpeed*FRACUNIT)
	local offsetX = 16*cos(mo.angle+ANGLE_90)
	local offsetY = 16*sin(mo.angle+ANGLE_90)
	P_TeleportMove(mo, mo.target.x+offsetX, mo.target.y+offsetY, mo.target.z)
end, MT_EMGSIDE)

addHook("MobjThinker", function(mo)
	if not mo.valid return end
	if not mo.target or not mo.target.valid P_RemoveMobj(mo) return end
	mo.angle = $+FixedAngle(rotSpeed*FRACUNIT)
	local topBottom = mo.top and 31*FRACUNIT or 1
	P_TeleportMove(mo, mo.target.x, mo.target.y, mo.target.z+topBottom)
end, MT_EMGTOP)

addHook("MobjThinker", function(mo)
	if not mo.valid return end
	if not mo.target or not mo.target.valid P_RemoveMobj(mo) return end
	mo.angle = $+FixedAngle(rotSpeed*FRACUNIT)
	P_TeleportMove(mo, mo.target.x, mo.target.y, mo.target.z+28*FRACUNIT)
end, MT_EMGRIB)
